package util.bd;

import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ConexaoBD {

    private Connection conexao;

    public ConexaoBD() {

        String dadosBanco = "jdbc:mysql://192.168.2.101/felipe";
        String usuario = "felipe";
        String senha = "123456";
        try {
            conexao = DriverManager.getConnection(dadosBanco, usuario, senha);

            System.out.println("Conectado com sucesso");

        } catch (SQLException ex) {
            System.err.println("Erro na conexão: " + ex.toString());
            Logger.getLogger(ConexaoBD.class.getName()).log(Level.SEVERE, null, ex);
        }


    }

    public ArrayList<ListaDeTurma> listarAlunosDaTurmaNaData(int tID, Date fData) {
        ArrayList<ListaDeTurma> lista = new ArrayList<ListaDeTurma>();

        String sql = "SELECT m.mID, a.aNome, f.fNumeroDeFaltas FROM matricula m,falta f,aluno a WHERE m.tID = ? AND m.aID = a.aID AND m.mId = f.mID AND f.fData = ?;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setInt(1, tID);
            st.setDate(2, fData);

            // executando a instrução
            ResultSet rs = st.executeQuery();

            // percorrendo a lista obtida do banco
            while (rs.next()) {
                ListaDeTurma lt = new ListaDeTurma();

                lt.setmID(new Long(rs.getInt("mID")));
                lt.setNomeAluno(rs.getString("aNome"));
                lt.setNumeroDeFaltas(rs.getInt("fNumeroDeFaltas"));

                // adicionando no ArrayList
                lista.add(lt);

            }

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na listagem de turma: " + ex.toString());
        }

        return lista;
    }

    public ArrayList<ListaDeTurma> listarAlunosDaTurma(int tID) {
        ArrayList<ListaDeTurma> lista = new ArrayList<ListaDeTurma>();

        String sql = "SELECT m.mID, a.aNome FROM matricula m, aluno a WHERE m.tID = ? AND m.aID = a.aID;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setInt(1, tID);

            // executando a instrução
            ResultSet rs = st.executeQuery();

            // percorrendo a lista obtida do banco
            while (rs.next()) {
                ListaDeTurma lt = new ListaDeTurma();

                lt.setmID(new Long(rs.getInt("mID")));
                lt.setNomeAluno(rs.getString("aNome"));
                lt.setNumeroDeFaltas(0);

                // adicionando no ArrayList
                lista.add(lt);

            }

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na listagem de turma: " + ex.toString());
        }

        return lista;
    }

    public void alterarFalta(int mID, Date fData,int fNumeroDeFaltas) {
        String sql = "UPDATE `felipe`.`falta` SET fNumeroDeFaltas = ? WHERE mID = ? AND fData = ?;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setInt(1, fNumeroDeFaltas);
            st.setInt(2, mID);
            st.setDate(3, fData);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na alteracao de falta: " + ex.toString());
        }
    }

    public void adicionarFalta(int mID, Date fData,int fNumeroDeFaltas) {
        String sql = "INSERT INTO `felipe`.`falta` (`mID`, `fData`, `fNumeroDeFaltas`) VALUES (?, ?, ?);";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setInt(1, mID);
            st.setDate(2, fData);
            st.setInt(3, fNumeroDeFaltas);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na adicao de falta: " + ex.toString());
        }
    }

    public ArrayList<ListaDeAlunos> listarAlunos() {
        ArrayList<ListaDeAlunos> lista = new ArrayList<ListaDeAlunos>();

        String sql = "SELECT aNome FROM aluno a";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            // executando a instrução
            ResultSet rs = st.executeQuery();

            // percorrendo a lista obtida do banco
            while (rs.next()) {
                ListaDeAlunos la = new ListaDeAlunos();

                la.setNomeAluno(rs.getString("aNome"));

                // adicionando no ArrayList
                lista.add(la);

            }

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na listagem de alunos: " + ex.toString());
        }

        return lista;
    }

    public void adicionarAluno(String aNome) {
        String sql = "INSERT INTO `felipe`.`aluno` (`aNome`) VALUES (?);";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setString(1, aNome);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na adicao do aluno: " + ex.toString());
        }
    }

    public void removerAluno(String aNome) {
        String sql = "DELETE FROM `felipe`.`aluno` WHERE `aNome`=?;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setString(1, aNome);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na remocao do aluno: " + ex.toString());
        }
    }

    public ArrayList<ListaDeDisciplinas> listarDisciplinas() {
        ArrayList<ListaDeDisciplinas> lista = new ArrayList<ListaDeDisciplinas>();

        String sql = "SELECT dNome FROM disciplina d";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            // executando a instrução
            ResultSet rs = st.executeQuery();

            // percorrendo a lista obtida do banco
            while (rs.next()) {
                ListaDeDisciplinas ld = new ListaDeDisciplinas();

                ld.setNomeDisciplina(rs.getString("dNome"));

                // adicionando no ArrayList
                lista.add(ld);

            }

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na listagem de disciplinas: " + ex.toString());
        }

        return lista;
    }

    public ArrayList<ListaDeProfessores> listarProfessores() {
        ArrayList<ListaDeProfessores> lista = new ArrayList<ListaDeProfessores>();

        String sql = "SELECT pNome, pPermissao, pSenha FROM professor p";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            // executando a instrução
            ResultSet rs = st.executeQuery();

            // percorrendo a lista obtida do banco
            while (rs.next()) {
                ListaDeProfessores lp = new ListaDeProfessores();

                lp.setNomeProfessor(rs.getString("pNome"));
                lp.setPermissao(rs.getLong("pPermissao"));
                lp.setSenha(rs.getString("pSenha"));

                // adicionando no ArrayList
                lista.add(lp);

            }

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na listagem de professores: " + ex.toString());
        }

        return lista;
    }

    public void adicionarDisciplina(String dNome) {
        String sql = "INSERT INTO `felipe`.`disciplina` (`dNome`) VALUES (?);";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setString(1, dNome);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na adicao da disciplina: " + ex.toString());
        }
    }

    public void removerDisciplina(String dNome) {
        String sql = "DELETE FROM `felipe`.`disciplina` WHERE `dNome`=?;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setString(1, dNome);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na remocao da disciplina: " + ex.toString());
        }
    }

    public void adicionarProfessor(String pNome, char[] pSenha, int pPermissao) {
        String sql = "INSERT INTO `felipe`.`professor` (`pNome`, `pSenha`, `pPermissao`) VALUES (?, ?, ?);";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            String pSenhaf = String.copyValueOf(pSenha);

            //prenchendo as ?
            st.setString(1, pNome);
            st.setString(2, pSenhaf);
            st.setInt(3, pPermissao);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na adicao do professor " + ex.toString());
        }
    }

    public void removerProfessor(String pNome) {
        String sql = "DELETE FROM `felipe`.`professor` WHERE `pNome`=?;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setString(1, pNome);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na remocao do professor: " + ex.toString());
        }
    }

    public ArrayList<ListaDeMatriculas> listarMatriculas() {
        ArrayList<ListaDeMatriculas> lista = new ArrayList<ListaDeMatriculas>();

        String sql = "SELECT mID, t.tID, aNome, dNome, pNome FROM matricula m, aluno a, turma t, disciplina d, professor p WHERE m.tID = t.tID AND t.dID = d.dID AND t.pID = p.pID AND m.aID = a.aID;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            // executando a instrução
            ResultSet rs = st.executeQuery();

            // percorrendo a lista obtida do banco
            while (rs.next()) {
                ListaDeMatriculas lm = new ListaDeMatriculas();

                lm.setmID(rs.getLong("mID"));
                lm.settID(rs.getLong("tID"));
                lm.setNomeAluno(rs.getString("aNome"));
                lm.setNomeDisciplina(rs.getString("dNome"));
                lm.setNomeProfessor(rs.getString("pNome"));

                // adicionando no ArrayList
                lista.add(lm);

            }

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na listagem de matriculas: " + ex.toString());
        }

        return lista;
    }

    public ArrayList<ListaDeTurmas> listarTurmas() {
        ArrayList<ListaDeTurmas> lista = new ArrayList<ListaDeTurmas>();

        String sql = "SELECT t.tID, p.pNome, d.dNome FROM professor p, disciplina d, turma t WHERE t.pID = p.pID AND t.dID = d.dID;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            // executando a instrução
            ResultSet rs = st.executeQuery();

            // percorrendo a lista obtida do banco
            while (rs.next()) {
                ListaDeTurmas lt = new ListaDeTurmas();

                lt.settID(rs.getLong("tID"));
                lt.setNomeDisciplina(rs.getString("dNome"));
                lt.setNomeProfessor(rs.getString("pNome"));

                // adicionando no ArrayList
                lista.add(lt);

            }

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na listagem de matriculas: " + ex.toString());
        }

        return lista;
    }

    public void adicionarMatricula(String aNome, int tID) {
        String sql = "SELECT a.aID FROM aluno a WHERE a.aNome = ?;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setString(1, aNome);

            // executando a instrução
            ResultSet rs = st.executeQuery();
            rs.next();
            int aID = rs.getInt("aID");

            sql = "INSERT INTO `felipe`.`matricula` (`aID`,`tID`) VALUES (?, ?);";
            st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setInt(1, aID);
            st.setInt(2, tID);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na adicao da matricula: " + ex.toString());
        }
    }

    public void removerMatricula(int mID) {
        String sql = "DELETE FROM `felipe`.`matricula` WHERE `mID`=?;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setInt(1, mID);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na remocao da matricula: " + ex.toString());
        }
    }

    public void adicionarTurma(String dNome, String pNome) {
        String sql = "SELECT d.dID FROM disciplina d WHERE d.dNome = ?;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setString(1, dNome);

            // executando a instrução
            ResultSet rs = st.executeQuery();
            rs.next();
            int dID = rs.getInt("dID");

            sql = "SELECT p.pID FROM professor p WHERE p.pNome = ?;";
            st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setString(1, pNome);

            // executando a instrução
            rs = st.executeQuery();
            rs.next();
            int pID = rs.getInt("pID");

            sql = "INSERT INTO `felipe`.`turma` (`dID`,`pID`) VALUES (?, ?);";
            st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setInt(1, dID);
            st.setInt(2, pID);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na adicao da turma: " + ex.toString());
        }
    }

    public void removerTurma(int tID) {
        String sql = "DELETE FROM `felipe`.`turma` WHERE `tID`=?;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setInt(1, tID);

            // executando a instrução
            st.execute();

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na remocao da turma: " + ex.toString());
        }
    }

    public ArrayList<ListaDeDatas> listarData(int tID) {
        ArrayList<ListaDeDatas> lista = new ArrayList<ListaDeDatas>();

        String sql = "SELECT fData FROM falta f, matricula m WHERE m.tID = ? AND m.mID = f.mID;";
        try {
            PreparedStatement st = conexao.prepareStatement(sql);

            //prenchendo as ?
            st.setInt(1, tID);

            // executando a instrução
            ResultSet rs = st.executeQuery();

            String data = new String();
            // percorrendo a lista obtida do banco
            while (rs.next()) {
                ListaDeDatas lda = new ListaDeDatas();

                String[] dataf = rs.getDate("fData").toString().split("-");
                data = dataf[2] + "/" + dataf[1] + "/" + dataf[0];
                if (lista.contains(data) == true){
                } else {
                    lda.setData(dataf[2] + "/" + dataf[1] + "/" + dataf[0]);
                }

                // adicionando no ArrayList
                lista.add(lda);

            }

            // fechando
            st.close();

        } catch (SQLException ex) {
            System.err.println("Erro na listagem de datas: " + ex.toString());
        }

        return lista;
    }
}
